# Generated by Django 2.2.7 on 2020-02-26 02:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('inputflow', '0003_auto_20191011_0015'),
    ]

    operations = [
        migrations.AddField(
            model_name='input',
            name='raw_content_type',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Raw content type'),
        ),
        migrations.AlterField(
            model_name='input',
            name='format',
            field=models.CharField(choices=[('form', 'Form'), ('multipart', 'Multipart form'), ('json', 'JSON')], default='json', max_length=60, verbose_name='Default format'),
        ),
        migrations.AlterField(
            model_name='inputsettings',
            name='default_format',
            field=models.CharField(choices=[('form', 'Form'), ('multipart', 'Multipart form'), ('json', 'JSON')], default='json', max_length=60, verbose_name='Default format'),
        ),
    ]



import threading
import pprint
from django.utils.translation import ugettext_lazy as _, ugettext
from django.contrib import admin
from django.forms.widgets import Textarea, TextInput
from adminsortable.admin import NonSortableParentAdmin, SortableTabularInline
from . import models
from django.db.models import TextField, CharField


class ModelAdminRequestMixin(admin.ModelAdmin):
    def __init__(self, *args, **kwargs):
        self._request_local = threading.local()
        self._request_local.request = None
        super(ModelAdminRequestMixin, self).__init__(*args, **kwargs)

    def get_request(self):
        return self._request_local.request

    def set_request(self, request):
        self._request_local.request = request

    def changeform_view(self, request, *args, **kwargs):
        self.set_request(request)
        return super(ModelAdminRequestMixin, self).changeform_view(request, *args, **kwargs)

    def add_view(self, request, *args, **kwargs):
        self.set_request(request)
        return super(ModelAdminRequestMixin, self).add_view(request, *args, **kwargs)

    def change_view(self, request, *args, **kwargs):
        self.set_request(request)
        return super(ModelAdminRequestMixin, self).change_view(request, *args, **kwargs)

    def changelist_view(self, request, *args, **kwargs):
        self.set_request(request)
        return super(ModelAdminRequestMixin, self).changelist_view(request, *args, **kwargs)

    def delete_view(self, request, *args, **kwargs):
        self.set_request(request)
        return super(ModelAdminRequestMixin, self).delete_view(request, *args, **kwargs)

    def history_view(self, request, *args, **kwargs):
        self.set_request(request)
        return super(ModelAdminRequestMixin, self).history_view(request, *args, **kwargs)

class TextWidget(Textarea):
    def __init__(self, attrs=None):
        default_attrs = {'cols': '26', 'rows': '3'}
        if attrs:
            default_attrs.update(attrs)
        super(TextWidget, self).__init__(default_attrs)


class CharsWidget(TextInput):
    def __init__(self, attrs=None):
        default_attrs = {'size': '18'}
        if attrs:
            default_attrs.update(attrs)
        super(CharsWidget, self).__init__(default_attrs)


class InputSettingsFieldInline(SortableTabularInline):
    model = models.InputSettingsField
    fields = ('input_name', 'output_name', 'date_format', 'default_value',
        'exclude_if_empty', 'omit', 'example_value')
    formfield_overrides = {
        TextField: {'widget': TextWidget},
        CharField: {'widget': CharsWidget},
    }


class InputSettingsAdmin(NonSortableParentAdmin):
    list_display = ('name', 'default_format', 'uid')
    readonly_fields = ('uid',)
    fields = ('uid', 'name', 'description', 'default_format')
    inlines = [InputSettingsFieldInline]
    actions = ['update_field_definitions']

    def get_fields(self, request, obj=None):
        if obj is None:
            return ('name', 'description', 'default_format')
        return ('uid', 'name', 'description', 'default_format')

    def update_field_definitions(self, request, queryset):
        for settings in queryset:
            settings.update_field_definitions()
        self.message_user(request, ugettext("Updated field definitions"))
    update_field_definitions.short_description = _("Update field definitions")


class InputAdmin(admin.ModelAdmin):
    list_display = ('settings', 'format', 'internal_source', 'processed',
        'modified', 'raw_content')
    list_filter = ('format', 'internal_source', 'processed', 'created',
        'modified', 'settings')
    actions = ['update_field_definitions', 'notify_input']
    
    def get_fields(self, request, obj=None):
        if obj is None:
            return ('settings', 'format', 'processed', 'raw_content', 'raw_content_type')
        return ('settings', 'internal_source', 'format', 'processed', 'raw_content',
            'raw_content_type', 'created', 'modified', 'data')

    def get_readonly_fields(self, request, obj=None):
        if obj is None:
            return []
        return self.get_fields(request, obj)

    def update_field_definitions(self, request, queryset):
        for settings in queryset:
            settings.update_field_definitions()
        self.message_user(request, ugettext("Updated field definitions"))
    update_field_definitions.short_description = _("Update field definitions")

    def data(self, obj=None):
        if obj is None:
            return None
        return pprint.pformat(obj.get_data(), indent=4)

    def notify_input(self, request, queryset):
        for input in queryset:
            input.notify()
        self.message_user(request, ugettext("Inputs notified."))
    notify_input.short_description = _("Notify inputs")


class WebhookAdmin(ModelAdminRequestMixin):
    list_display = ('uid', 'name', 'settings', 'url')
    fields = ('uid', 'name', 'description', 'settings', 'url')
    readonly_fields = ('uid', 'url')

    def url(self, obj=None):
        if obj is None:
            return None
        return obj.get_webhook_url(self.get_request())


admin.site.register(models.InputSettings, InputSettingsAdmin)
admin.site.register(models.Input, InputAdmin)
admin.site.register(models.Webhook, WebhookAdmin)



# -*- coding: utf-8
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

class InputFlowConfig(AppConfig):
    name = 'inputflow'
    varbose_name = _("Input flow")



# -*- coding: utf-8 -*-
from django.conf.urls import url
from . import views


app_name = 'inputflow'


urlpatterns = [
    url(r'^webhook/(?P<uid>[0-9a-fA-F\-]+)$', views.webhook, name='inputflow-webhook'),
    url(r'^webhook/(?P<uid>[0-9a-fA-F\-]+)/$', views.webhook, name='inputflow-webhook'),
]



from prettytable import PrettyTable
from django.core.management.base import BaseCommand
import six
from ...models import InputSettingsField


class Command(BaseCommand):
    def handle(self, *args, **kwargs):
        table = PrettyTable()
        table.field_names = ["settings", "input_name", "output_name", "date_format", "default_value", "omit_empty", "omit"]
        for field in InputSettingsField.objects.order_by('settings', 'field_position'):
            table.add_row([field.settings, field.input_name, field.output_name, 
                field.date_format, field.default_value, field.exclude_if_empty, field.omit])
        six.print_(table)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import datetime
from collections import OrderedDict
import uuid
import json
from django.utils.translation import ugettext_lazy as _
from django.utils.six import python_2_unicode_compatible
from django.utils.six.moves.urllib.parse import parse_qs
from django.urls import reverse
from django.db import models
import django.dispatch
from adminsortable.models import SortableMixin
from adminsortable.fields import SortableForeignKey
from .utils import Utils


input_notification = django.dispatch.Signal(providing_args=['input_data', 'input_type', 'input_id'])


@python_2_unicode_compatible
class InputSettings(models.Model):
    FORMATS = [
        ('form', _("Form")),
        ('multipart', _("Multipart form")),
        ('json', _("JSON"))
    ]
    uid = models.UUIDField(_("UID"), max_length=60, unique=True, default=uuid.uuid4,
        editable=False)
    name = models.CharField(_("Name"), max_length=60, unique=True)
    description = models.TextField(_("Description"), blank=True, default='')
    default_format = models.CharField(_("Default format"), max_length=60,
        default='json', choices=FORMATS)

    class Meta:
        verbose_name = _("Input settings")
        verbose_name_plural = _("Input settings")

    def __str__(self):
        return self.name

    def update_field_definitions(self):
        """Updates input settings field definitions from the last created
        input of this settings.
        """
        input = self.inputs.last()
        if input:
            input.update_field_definitions()
            return input
        return False


@python_2_unicode_compatible
class InputSettingsField(SortableMixin):
    # orderable info
    settings = SortableForeignKey(InputSettings, verbose_name=_("Input settings"),
        related_name='input_fields', on_delete=models.CASCADE)
    field_position = models.PositiveIntegerField(default=0, editable=False, db_index=True)
    # transformation info
    input_name = models.CharField(_("Input name"), max_length=60)
    output_name = models.CharField(_("Field name"), max_length=60, blank=True, default='')
    example_value = models.TextField(_("Example value"), blank=True, default='')
    date_format = models.CharField(_("Date format"), max_length=40, blank=True, default='')
    default_value = models.TextField(_("Default value"), blank=True, default='')
    exclude_if_empty = models.BooleanField(_("Exclude if empty"), blank=True, default=False)
    omit = models.BooleanField(_("Omit"), blank=True, default=False)

    class Meta:
        verbose_name = _("Input field")
        verbose_name_plural = _("Input fields")
        ordering = ['field_position']
        unique_together = [('settings', 'input_name')]

    def __str__(self):
        return self.input_name

class Input(models.Model):
    "Input data."
    settings = models.ForeignKey(InputSettings, verbose_name=_("Settings"),
        blank=False, null=False, on_delete=models.PROTECT,
        related_name='inputs')
    internal_source = models.BooleanField(_("Internal source"), blank=True,
        default=True)
    format = models.CharField(_("Default format"), max_length=60, default='json',
        choices=InputSettings.FORMATS)
    processed = models.BooleanField(_("Processed"), blank=True, default=False)
    raw_content = models.TextField(_("Raw content"))
    raw_content_type = models.CharField(_("Raw content type"), max_length=250, blank=True, default='')
    created = models.DateTimeField(_("Created"), auto_now_add=True, blank=True, null=True)
    modified = models.DateTimeField(_("Modified"), auto_now=True, blank=True, null=True)

    class Meta:
        verbose_name = _("Input")
        verbose_name_plural = _("Inputs")

    def get_raw_dict(self):
        if self.format == 'json':
            return json.loads(self.raw_content)
        if self.format == 'form':
            result = parse_qs(self.raw_content)
            return dict((key, value[0]) for key, value in result.items())
        raise ValueError("Input format not supported: {}".format(self.format))

    def update_field_definitions(self):
        for key, value in self.get_raw_dict().items():
            try:
                field = InputSettingsField.objects.get(settings=self.settings,
                    input_name=key)
                field.example_value = str(value)
                field.save()
            except InputSettingsField.DoesNotExist:
                field = InputSettingsField()
                field.settings = self.settings
                field.input_name = key
                field.example_value = str(value)
                field.save()

    def get_data(self):
        raw_dict = self.get_raw_dict()
        if self.settings.input_fields.all().count() == 0:
            return raw_dict
        result = OrderedDict()
        for field in InputSettingsField.objects.filter(settings=self.settings):
            if field.omit:
                continue
            if field.input_name in raw_dict:
                value = raw_dict[field.input_name]
                if field.date_format:
                    value = datetime.datetime.strptime(value, field.date_format)
                Utils.write_to_dict(result, field.output_name or field.input_name, value)
            elif not field.exclude_if_empty:
                Utils.write_to_dict(result, field.output_name or field.input_name,
                    field.default_value or None)
        return result

    def notify(self):
        input_notification.send_robust(sender=self.__class__, input_data=self.get_data(),
            input_type=self.settings.uid, input_id=self.pk)



@python_2_unicode_compatible
class Webhook(models.Model):
    uid = models.UUIDField(_("UID"), max_length=60, unique=True, default=uuid.uuid4,
        editable=False)
    name = models.CharField(_("Name"), max_length=60, unique=True)
    description = models.TextField(_("Description"), blank=True, default='')
    settings = models.ForeignKey(InputSettings, verbose_name=_("Settings"),
        blank=False, null=False, on_delete=models.CASCADE, related_name='webhooks')

    class Meta:
        verbose_name = _("Webhook")
        verbose_name_plural = _("Webhooks")

    def __str__(self):
        return self.name

    def get_webhook_url(self, request=None):
        url = reverse('inputflow:inputflow-webhook', args=(self.uid,))
        return url if request is None else request.build_absolute_uri(url)



import json
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_exempt
from django.http import HttpResponse
from . import models


@csrf_exempt
def webhook(request, uid):
    if request.method != 'POST':
        return HttpResponseBadRequest("Method not allowed.")
    webhook = get_object_or_404(models.Webhook, uid=uid)
    input = models.Input()
    input.settings =  webhook.settings
    input.internal_source = False
    
    input.raw_content = request.body.decode('utf-8')
    input.raw_content_type = request.META['CONTENT_TYPE']
    if request.META['CONTENT_TYPE'].startswith('application/x-www-form-urlencoded'):
        input.format = 'form'
    elif request.META['CONTENT_TYPE'].startswith('multipart/form-data'):
        input.raw_content = json.dumps(request.POST.dict())
        input.raw_content_type = ''
        input.format = 'json'
    else:
        input.format = 'json'
    input.save()
    input.notify()
    return HttpResponse("Ok")



# Generated by Django 2.2.6 on 2019-10-10 21:12

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('inputflow', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Webhook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='UID')),
                ('name', models.CharField(max_length=60, unique=True, verbose_name='Name')),
                ('description', models.TextField(blank=True, default='', verbose_name='Description')),
                ('settings', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='webhooks', to='inputflow.InputSettings', verbose_name='Settings')),
            ],
            options={
                'verbose_name': 'Webhook',
                'verbose_name_plural': 'Webhooks',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-10-11 03:15
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('inputflow', '0002_webhook'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webhook',
            name='settings',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='webhooks', to='inputflow.InputSettings', verbose_name='Settings'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-10 20:35

import adminsortable.fields
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='InputSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='UID')),
                ('name', models.CharField(max_length=60, unique=True, verbose_name='Name')),
                ('description', models.TextField(blank=True, default='', verbose_name='Description')),
                ('default_format', models.CharField(choices=[('form', 'Form'), ('json', 'JSON')], default='json', max_length=60, verbose_name='Default format')),
            ],
            options={
                'verbose_name': 'Input settings',
                'verbose_name_plural': 'Input settings',
            },
        ),
        migrations.CreateModel(
            name='Input',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('internal_source', models.BooleanField(blank=True, default=True, verbose_name='Internal source')),
                ('format', models.CharField(choices=[('form', 'Form'), ('json', 'JSON')], default='json', max_length=60, verbose_name='Default format')),
                ('processed', models.BooleanField(blank=True, default=False, verbose_name='Processed')),
                ('raw_content', models.TextField(verbose_name='Raw content')),
                ('created', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Created')),
                ('modified', models.DateTimeField(auto_now=True, null=True, verbose_name='Modified')),
                ('settings', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='inputs', to='inputflow.InputSettings', verbose_name='Settings')),
            ],
            options={
                'verbose_name': 'Input',
                'verbose_name_plural': 'Inputs',
            },
        ),
        migrations.CreateModel(
            name='InputSettingsField',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field_position', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('input_name', models.CharField(max_length=60, verbose_name='Input name')),
                ('output_name', models.CharField(blank=True, default='', max_length=60, verbose_name='Field name')),
                ('example_value', models.TextField(blank=True, default='', verbose_name='Example value')),
                ('date_format', models.CharField(blank=True, default='', max_length=40, verbose_name='Date format')),
                ('default_value', models.TextField(blank=True, default='', verbose_name='Default value')),
                ('exclude_if_empty', models.BooleanField(blank=True, default=False, verbose_name='Exclude if empty')),
                ('omit', models.BooleanField(blank=True, default=False, verbose_name='Omit')),
                ('settings', adminsortable.fields.SortableForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='input_fields', to='inputflow.InputSettings', verbose_name='Input settings')),
            ],
            options={
                'verbose_name': 'Input field',
                'verbose_name_plural': 'Input fields',
                'ordering': ['field_position'],
                'unique_together': {('settings', 'input_name')},
            },
        ),
    ]
